// Copyright 2025 "Johan Maasing" <johan@zoom.nu>
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
grammar Endpoints;
document                : generatorconfig? (namedTypeDeclaration|endpoint)* ;
generatorconfig         : '{' (configitem)? (',' configitem)* '}';
configitem              : configkey ':' configvalue ;
configkey               : IDENTIFIER ;
configvalue             : (IDENTIFIER|VALUE) ;
namedTypeDeclaration    : typeName typeDeclaration ;
typeName                : IDENTIFIER ;
typeDeclaration         : '(' typeField (',' typeField)* ')' ;
typeField               : fieldName ':' fieldType ;
fieldName               : IDENTIFIER ;
fieldType               : IDENTIFIER ;
requestBody             : REQUEST_PREFIX  (namedTypeDeclaration | typeDeclaration | IDENTIFIER) ;
responseBody            : RESPONSE_PREFIX (namedTypeDeclaration | typeDeclaration | IDENTIFIER) ;
endpoint                : path requestBody responseBody?;
path                    : (pathSegment)+ ;
pathSegment             : SLASH (IDENTIFIER|VALUE) ;


fragment DIGIT          : [0-9] ;
fragment LOWERCASE      : [a-z] ;
fragment UPPERCASE      : [A-Z] ;
fragment GENERICS       : '['|']'|'<'|'>' ;
fragment DOT            : '.' ;
fragment COMMENT_BEGIN  : '/*' ;
fragment COMMENT_END    : '*/' ;
WS                      : [ \t\n\r]+ -> skip;
COMMENT                 : COMMENT_BEGIN .*? COMMENT_END -> skip;
REQUEST_PREFIX          : '<-' ;
RESPONSE_PREFIX         : '->' ;
SLASH                   : '/' ;
IDENTIFIER              : (LOWERCASE | UPPERCASE) (LOWERCASE | UPPERCASE | DIGIT | GENERICS | DOT)* ;
VALUE                   : ~[ ,{}:()/="#';*\n\r\t]+  ;
